==> spaceships-1.0.0-linux-x86_64.tar.xz: spaceships/README.md <==
# Spaceships
Spaceships is a [free-as-in-freedom](https://www.gnu.org/philosophy/free-sw.html) multiplayer game that's written in Rust and created by Marcus Del Favero. Note that this game is currently in **heavy development**, and isn't the most user-friendly, so don't get your hopes too high.

## Links
* Website: [https://spaceships.me](https://spaceships.me)
* Download: [https://spaceships.me/download](https://spaceships.me/download)
* Source code: [https://codeberg.org/rustydev/spaceships](https://codeberg.org/rustydev/spaceships)

## Screenshots
![Red player killing the blue player.](https://spaceships.me/screenshots/game1.png)
![Blue player shooting the red player.](https://spaceships.me/screenshots/game2.png)

## How to play
### Joining a game
To start a game, you first need to start a server or join an already existing server. See the "Hosting a server" section for more information about starting a server.

If the game server is hosted on the local network, joining the game is done by clicking "Play" and then "Local" under. Then click "Search LAN" and if everything works well, the game server will pop up and you can then click "Play".

If the game server is hosted on the internet (or on the local network but the above instructions don't work for some reason), instead click "Enter Address" and enter the IP address or domain name in the "Host" section. The "Game ID" section can be left blank unless the server administrator tells you to set it to a particular value.

You can also play Spaceships in a singleplayer mode where you can immediately join a local game (where nobody else can join) to test things out. Playing Spaceships only makes sense with other people, it won't be fun at all doing it by yourself (this might change in the future if I create bot players).

### Playing a game
The goal of the game is simple, you're a spaceship and you need to kill other spaceships by firing bullets at them to diminish their health and eventually destroy them. At the moment there isn't anything beyond that. The server keeps track of a list of scores, which are the number of kills each player has. If you leave the game, your score will be reset.

#### Controls
To move around, use WASD or the arrow keys. To change the direction you're looking in and where bullets will be fired from, move the mouse. Bullets are shot by pressing the left mouse button or spacebar. You can hold down these buttons to easily fire multiple bullets at a time. These controls can be changed by pressing the "Settings" button while in the title screen.

#### Health
Your health is displayed in the health bar at the bottom of your screen, with the number of bullets also displayed above that. The health bars of other players are displayed right above them. After getting shot, players slowly regenerate their health.

#### Ammo
You have a finite amount of bullets (unless the server is configured to provide you infinite), which are reset when you die. Periodically during the game, ammo crates are spawned which supply the player collecting them with a random amount of bullets.

#### Powerups
Additionally, powerups are periodically spawned which give the player collecting them a temporary effect. These effects either last for 30 seconds or happen immediately (teleportation) and are listed as follows:
1. **Speed.** This is the green powerup which increases your movement speed.
2. **Reload.** This is the yellow powerup which increases your reload speed, making you shoot faster.
3. **Health.** This is the pink powerup which increases your health and makes you regenerate health faster. The increase of health that this powerup provides can increase your health above the usual limit and make your health bar blue.
4. **Damage.** This is the red powerup which increases the damage of your bullets.
5. **Forcefield.** This is the blue powerup which provides a forcefield that deflects bullets fired by other players.
6. **Teleportation.** This is the purple powerup which teleports you to a random place on the map. Unlike the others, this can be disadvantageous depending where you teleport...

Here's a screenshot of these powerups:
![Powerups](https://spaceships.me/screenshots/powerups.png)

## Hosting a server
Right now hosting a server is only possible using the command line.

There are many different worlds that you can create, a few of them are provided in the `configs/` directory. Feel free to modify them or create your own. I haven't yet wrote documentation on how exactly things are configured, see the source code (and in particular `src/net/server/config.rs`) for more information.

To start a server, simply run `./spaceships server configs/CONFIG.ron`, replacing `configs/CONFIG.ron` with the exact configuration to use. On Windows, you would run `./spaceships.exe server ...`.

The server program outputs logs to the terminal (in particular, stderr), which include information about players joining and leaving the game. The server also provides a simple REPL interface which allows more advanced configuration that I haven't yet documented.

If you're running an internet server with an associated domain name, if users want to connect to the server by entering the domain name you will need a valid TLS (also called SSL) certificate. To run the server to use this certificate, run `./spaceships ... --cert-path PATH --key-path PATH`, replacing `PATH` with the paths of the certificate and private key. These paths are to files in the PEM format.

## Requesting server info
This game is licensed under the GNU AGPLv3, which requires that any user interacting with the game over the network can get a copy of the exact version of the game running on the server. If you want to know the exact version of a server and if it's a fork, run `./spaceships info HOST`, where `HOST` is the domain name or IP address of the server.

# Installation
This game is available on both Linux and Windows. It might also be possible to compile to macOS but I haven't yet tested it. Go to [https://spaceships.me/download](https://spaceships.me/download) to download a copy. You can also build from source.

# Building
Here are the steps needed to build the game. I recommend building this game using the latest version of Rust, as that's the only version I will bother testing.

## Features
This project has two Cargo features, `client` and `standalone`, with just `client` being enabled by default.
* `client`: This builds the game client as well as the game server. If you want to create a server-only build (which can save disk space if you're just interested in running a server), disable this feature by running `cargo build --release --no-default-features` at step 2. If you do disable this feature, you can skip the first step of building the assets during compilation.
* `standalone`: This is disabled by default but is enabled when building releases. When this flag is set, the path to the `data` directory (which stores all assets used by the client) is based on the path of the Spaceships executable. This makes it convenient to play the game as a regular user as you don't need to worry about the current working directory being correct. However this is annoying during development as the binary is located in `target/debug/` or `target/release/`, which would expect the `data/` directory to be in this target directory, so it is disabled by default. To build using this flag, run `cargo build --release -F standalone` when at step 2.

## Steps
1. **Build the assets.** Most assets used in this game are SVG files located in `assets/data/`, with an exception for the powerup textures and explosion effects (not an image) which are procedurally generated. These assets must be generated/converted to PNGs during the build process. To do this, go into the `assets` directory and then run `cargo run`. This will take a few seconds. **Note that this won't work on Windows due to lazy programming on my part (writing temporary files to /tmp) and requires installing Inkscape.** If you want to skip this step, you can instead download a binary release of this game and copy the files from `data/textures/` and `data/explosions`.
2. **Build the game.** Now the game can be built by running `cargo build --release`.

# Licensing and contributing
**Spaceships is free software**, released under the GNU AGPLv3 license (but not at any later versions) with an exception that is explained later. If you contribute to this project, you own the copyright of your changes (no [CLA bullshit](https://en.wikipedia.org/wiki/Contributor_License_Agreement#Relicensing_controversy) here) but you must release your work under a software license compatible with the GNU AGPLv3 with this provided exception.

The contents of the GNU AGPLv3 license is provided in the attached `LICENSE` file and also [available online](https://www.gnu.org/licenses/agpl-3.0.en.html).

## Fonts used
This game uses the [Inter font](https://rsms.me/inter) (stored at `data/fonts/inter.otf`), which is released under the SIL Open Font License version 1.1. The license and copyright details are available in `data/fonts/inter.otf.license`. The exact variant of Inter used is Inter-Medium.otf modified with the tnum feature to be enabled (fixed width for all numbers). This is achieved by running `pyftfeatfreeze -f tnum old.otf new.otf`.

This game also uses the [Public Pixel Font](https://opengameart.org/content/public-pixel-font) (stored at `data/fonts/score_font.otf`) which is released under the CC0 license.

## Dependencies
Software licensing is a pain in the arse, especially when the project has hundreds of transitive dependencies (dependencies of dependencies) which is the case here in the Rust ecosystem. Luckily there exists the [cargo deny](https://github.com/EmbarkStudios/cargo-deny) Cargo plugin which automatically checks the licenses of all dependencies used. The dependencies that this project uses are listed in the `Cargo.lock` file in the source distribution of this project.

Running `cargo deny check licenses` in this project with the provided `deny.toml` file shows that everything's fine when it comes to licensing. Read the comments in `deny.toml` explaining why each of these individual allowed licenses are acceptable, though keep reading this for the explanation of the OpenSSL license.

Two of this project's transitive dependencies are `aws-lc-sys` and `ring`. Both of these are released under the OpenSSL license and many other permissive free software licenses. A problem with the OpenSSL license is that it is incompatible with the GNU GPL licenses. However, it is [still possible to use incompatible libraries](https://www.gnu.org/licenses/gpl-faq.en.html#GPLIncompatibleLibs) *as long as* the copyright holder chooses to provide an exception to allow these. Luckily I am the copyright holder and none of my dependencies use a GPL license so I can freely add such an exception.

Hopefully in the future this licensing problem will completely go away. The developer of `ring` [hopes this will be the case sometime](https://github.com/briansmith/ring/issues/1827), and I wish them the best of luck in doing so. I can completely replace `aws-lc-rs` with `ring` if I want to, in case `aws-lc-rs` doesn't change.

Here is this exception, which I created by cutting and modifying pieces of [this text from GNU's page about incompatible licenses](https://www.gnu.org/licenses/gpl-faq.en.html#GPLIncompatibleLibs) and [this text here](https://git.deluge-torrent.org/deluge/plain/LICENSE) at the beginning of Deluge's license.

## OpenSSL exception
Additional permission under GNU AGPLv3 section 7:

In addition, as a special exception, the copyright holder(s) give permission to link the code of portions of this program with any code released under the OpenSSL License that is either a part of OpenSSL or has originated from OpenSSL (which is including but not limited to the `ring` and `aws-lc-sys` crates that are found at https://crates.io). If you redistribute this project (either identical or modified copies), you are allowed to keep this exact same OpenSSL exception or remove it (in which the license would be identical to the unmodified GNU AGPLv3).

# Forking
**If you're forking this project, please modify the details in `Cargo.toml` to allow users requesting information about the server (see the "Requesting server info" section for more information) to know where to obtain a copy of the source code.**

==> spaceships-1.0.1-linux-x86_64.tar.xz: spaceships/README.md <==
# Spaceships
Spaceships is a [free-as-in-freedom](https://www.gnu.org/philosophy/free-sw.html) multiplayer game that's written in Rust and created by Marcus Del Favero. Note that this game is currently in **heavy development**, and isn't the most user-friendly, so don't get your hopes too high.

## Links
* Website: [https://spaceships.me](https://spaceships.me)
* Download: [https://spaceships.me/download](https://spaceships.me/download)
* Source code: [https://codeberg.org/rustydev/spaceships](https://codeberg.org/rustydev/spaceships)

## Screenshots
![Red player killing the blue player.](https://spaceships.me/screenshots/game1.png)
![Blue player shooting the red player.](https://spaceships.me/screenshots/game2.png)

## How to play
### Joining a game
To start a game, you first need to start a server or join an already existing server. See the "Hosting a server" section for more information about starting a server.

If the game server is hosted on the local network, joining the game is done by clicking "Play" and then "Local" under. Then click "Search LAN" and if everything works well, the game server will pop up and you can then click "Play".

If the game server is hosted on the internet (or on the local network but the above instructions don't work for some reason), instead click "Enter Address" and enter the IP address or domain name in the "Host" section. The "Game ID" section can be left blank unless the server administrator tells you to set it to a particular value.

You can also play Spaceships in a singleplayer mode where you can immediately join a local game (where nobody else can join) to test things out. Playing Spaceships only makes sense with other people, it won't be fun at all doing it by yourself (this might change in the future if I create bot players).

### Playing a game
The goal of the game is simple, you're a spaceship and you need to kill other spaceships by firing bullets at them to diminish their health and eventually destroy them. At the moment there isn't anything beyond that. The server keeps track of a list of scores, which are the number of kills each player has. If you leave the game, your score will be reset.

#### Controls
To move around, use WASD or the arrow keys. To change the direction you're looking in and where bullets will be fired from, move the mouse. Bullets are shot by pressing the left mouse button or spacebar. You can hold down these buttons to easily fire multiple bullets at a time. These controls can be changed by pressing the "Settings" button while in the title screen.

You can zoom the camera in and out by scrolling (or by pressing 'J' and 'K'), and you can reset to the default zoom by pressing the middle mouse button.

#### Health
Your health is displayed in the health bar at the bottom of your screen, with the number of bullets also displayed above that. The health bars of other players are displayed right above them. After getting shot, players slowly regenerate their health.

#### Ammo
You have a finite amount of bullets (unless the server is configured to provide you infinite), which are reset when you die. Periodically during the game, ammo crates are spawned which supply the player collecting them with a random amount of bullets.

#### Powerups
Additionally, powerups are periodically spawned which give the player collecting them a temporary effect. These effects either last for 30 seconds or happen immediately (teleportation) and are listed as follows:
1. **Speed.** This is the green powerup which increases your movement speed.
2. **Reload.** This is the yellow powerup which increases your reload speed, making you shoot faster.
3. **Health.** This is the pink powerup which increases your health and makes you regenerate health faster. The increase of health that this powerup provides can increase your health above the usual limit and make your health bar blue.
4. **Damage.** This is the red powerup which increases the damage of your bullets.
5. **Forcefield.** This is the blue powerup which provides a forcefield that deflects bullets fired by other players.
6. **Teleportation.** This is the purple powerup which teleports you to a random place on the map. Unlike the others, this can be disadvantageous depending where you teleport...

Here's a screenshot of these powerups:

![Powerups](https://spaceships.me/screenshots/powerups.png)

## Hosting a server
Right now hosting a server is only possible using the command line.

There are many different worlds that you can create, a few of them are provided in the `configs/` directory. Feel free to modify them or create your own. I haven't yet wrote documentation on how exactly things are configured, see the source code (and in particular `src/net/server/config.rs`) for more information.

To start a server, simply run `./spaceships server configs/CONFIG.ron`, replacing `configs/CONFIG.ron` with the exact configuration to use. On Windows, you would run `./spaceships.exe server ...`.

The server program outputs logs to the terminal (in particular, stderr), which include information about players joining and leaving the game. The server also provides a simple REPL interface which allows more advanced configuration that I haven't yet documented.

If you're running an internet server with an associated domain name, if users want to connect to the server by entering the domain name you will need a valid TLS (also called SSL) certificate. To run the server to use this certificate, run `./spaceships ... --cert-path PATH --key-path PATH`, replacing `PATH` with the paths of the certificate and private key. These paths are to files in the PEM format.

## Requesting server info
This game is licensed under the GNU AGPLv3, which requires that any user interacting with the game over the network can get a copy of the exact version of the game running on the server. If you want to know the exact version of a server and if it's a fork, run `./spaceships info HOST`, where `HOST` is the domain name or IP address of the server.

# Installation
This game is available on both Linux and Windows. It might also be possible to compile to macOS but I haven't yet tested it. Go to [https://spaceships.me/download](https://spaceships.me/download) to download a copy. You can also build from source.

# Building
Here are the steps needed to build the game. I recommend building this game using the latest version of Rust, as that's the only version I will bother testing.

## Features
This project has two Cargo features, `client` and `standalone`, with just `client` being enabled by default.
* `client`: This builds the game client as well as the game server. If you want to create a server-only build (which can save disk space if you're just interested in running a server), disable this feature by running `cargo build --release --no-default-features` at step 2. If you do disable this feature, you can skip the first step of building the assets during compilation.
* `standalone`: This is disabled by default but is enabled when building releases. When this flag is set, the path to the `data` directory (which stores all assets used by the client) is based on the path of the Spaceships executable. This makes it convenient to play the game as a regular user as you don't need to worry about the current working directory being correct. However this is annoying during development as the binary is located in `target/debug/` or `target/release/`, which would expect the `data/` directory to be in this target directory, so it is disabled by default. To build using this flag, run `cargo build --release -F standalone` when at step 2.

## Steps
1. **Build the assets.** Most assets used in this game are SVG files located in `assets/data/`, with an exception for the powerup textures and explosion effects (not an image) which are procedurally generated. These assets must be generated/converted to PNGs during the build process. To do this, go into the `assets` directory and then run `cargo run`. This will take a few seconds. **Note that this won't work on Windows due to lazy programming on my part (writing temporary files to /tmp) and requires installing Inkscape.** If you want to skip this step, you can instead download a binary release of this game and copy the files from `data/textures/` and `data/explosions`.
2. **Build the game.** Now the game can be built by running `cargo build --release`. **This will take a few minutes when compiling for the first time!**

# Licensing and contributing
**Spaceships is free software**, released under the GNU AGPLv3 license (but not at any later versions) with an exception that is explained later. If you contribute to this project, you own the copyright of your changes (no [CLA bullshit](https://en.wikipedia.org/wiki/Contributor_License_Agreement#Relicensing_controversy) here) but you must release your work under a software license compatible with the GNU AGPLv3 with this provided exception.

The contents of the GNU AGPLv3 license is provided in the attached `LICENSE` file and also [available online](https://www.gnu.org/licenses/agpl-3.0.en.html).

## Fonts used
This game uses the [Inter font](https://rsms.me/inter) (stored at `data/fonts/inter.otf`), which is released under the SIL Open Font License version 1.1. The license and copyright details are available in `data/fonts/inter.otf.license`. The exact variant of Inter used is Inter-Medium.otf modified with the tnum feature to be enabled (fixed width for all numbers). This is achieved by running `pyftfeatfreeze -f tnum old.otf new.otf`.

This game also uses the [Public Pixel Font](https://opengameart.org/content/public-pixel-font) (stored at `data/fonts/score_font.otf`) which is released under the CC0 license.

## Dependencies
Software licensing is a pain in the arse, especially when the project has hundreds of transitive dependencies (dependencies of dependencies) which is the case here in the Rust ecosystem. Luckily there exists the [cargo deny](https://github.com/EmbarkStudios/cargo-deny) Cargo plugin which automatically checks the licenses of all dependencies used. The dependencies that this project uses are listed in the `Cargo.lock` file in the source distribution of this project.

Running `cargo deny check licenses` in this project with the provided `deny.toml` file shows that everything's fine when it comes to licensing. Read the comments in `deny.toml` explaining why each of these individual allowed licenses are acceptable, though keep reading this for the explanation of the OpenSSL license.

Two of this project's transitive dependencies are `aws-lc-sys` and `ring`. Both of these are released under the OpenSSL license and many other permissive free software licenses. A problem with the OpenSSL license is that it is incompatible with the GNU GPL licenses. However, it is [still possible to use incompatible libraries](https://www.gnu.org/licenses/gpl-faq.en.html#GPLIncompatibleLibs) *as long as* the copyright holder chooses to provide an exception to allow these. Luckily I am the copyright holder and none of my dependencies use a GPL license so I can freely add such an exception.

Hopefully in the future this licensing problem will completely go away. The developer of `ring` [hopes this will be the case sometime](https://github.com/briansmith/ring/issues/1827), and I wish them the best of luck in doing so. I can completely replace `aws-lc-rs` with `ring` if I want to, in case `aws-lc-rs` doesn't change.

Here is this exception, which I created by cutting and modifying pieces of [this text from GNU's page about incompatible licenses](https://www.gnu.org/licenses/gpl-faq.en.html#GPLIncompatibleLibs) and [this text here](https://git.deluge-torrent.org/deluge/plain/LICENSE) at the beginning of Deluge's license.

## OpenSSL exception
Additional permission under GNU AGPLv3 section 7:

In addition, as a special exception, the copyright holder(s) give permission to link the code of portions of this program with any code released under the OpenSSL License that is either a part of OpenSSL or has originated from OpenSSL (which is including but not limited to the `ring` and `aws-lc-sys` crates that are found at https://crates.io). If you redistribute this project (either identical or modified copies), you are allowed to keep this exact same OpenSSL exception or remove it (in which the license would be identical to the unmodified GNU AGPLv3).

# Forking
**If you're forking this project, please modify the details in `Cargo.toml` to allow users requesting information about the server (see the "Requesting server info" section for more information) to know where to obtain a copy of the source code.**
